/*
 * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "sensor-config-dialog.h"
#include "sensors-applet-gconf-keys.h"

static void sensors_list_sensor_toggled(GtkCellRenderer *renderer, gchar *path_str, SensorsList *sensors_list) {
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	gboolean value;
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(sensors_list->list), &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list), &iter, ENABLE_COLUMN, &value, -1);
	
	value = !value;
	gtk_list_store_set(sensors_list->list, &iter, ENABLE_COLUMN, value, -1);
	
	gtk_tree_path_free(path);

	/* need to update entire display if a sensor has been added /
	   removed */
	sensors_applet_update_display(sensors_list->sensors_applet);
}

static void sensors_list_sensor_name_changed(GtkCellRenderer *renderer, gchar *path_str, gchar *new_text, SensorsList *sensors_list) {
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	
	gtk_tree_model_get_iter(GTK_TREE_MODEL(sensors_list->list), &iter, path);

	gtk_list_store_set(sensors_list->list, &iter, LABEL_COLUMN, new_text, -1);
	
	gtk_tree_path_free(path);

	/* update this lists labels */
	sensors_list->update_labels(sensors_list);

}

static void sensors_list_sensor_config_button_clicked(GtkButton *button, SensorsList *sensors_list) {
	sensor_config_dialog_create(button, sensors_list);
}

/* if a row is selected, make sure button is able to be clicked */
static void sensors_list_selection_changed(GtkTreeSelection *selection, SensorsList *sensors_list) {
	if (gtk_tree_selection_get_selected(selection, NULL, NULL)) {
		gtk_widget_set_sensitive(GTK_WIDGET(sensors_list->sensor_config_button), TRUE);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(sensors_list->sensor_config_button), FALSE);
	}
}


static void sensors_list_get_list_from_gconf(SensorsList *sensors_list, GSList **list, const gchar *key, GConfValueType list_type) {
	gchar *gconf_key;
	gconf_key = g_strdup_printf("%s/%s", sensors_list->id, key);
	*list = panel_applet_gconf_get_list(sensors_list->sensors_applet->applet, gconf_key, list_type, NULL);
	g_free(gconf_key);
}

static void sensors_list_set_list_in_gconf(SensorsList *sensors_list, GSList *list, const gchar *key, GConfValueType list_type) {
	gchar *gconf_key;
	gconf_key = g_strdup_printf("%s/%s", sensors_list->id, key);
	panel_applet_gconf_set_list(sensors_list->sensors_applet->applet,
				    gconf_key,
				    list_type,
				    list,
				    NULL);
	g_free(gconf_key);
}

static gboolean sensors_list_alarm_timeout(gchar *alarm_command) {
	gnome_execute_shell(NULL, alarm_command);
	return TRUE;
}

/* function to be called by specific sensor lists if an error occurs
   when loading values from gconf */
void sensors_list_gconf_error_occurred(const gchar *id) {

	gchar *markup;
	GtkDialog *dialog;
	GtkHBox *hbox;
	GtkImage *icon;
	GtkLabel *label;

	dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(_(PACKAGE_NAME " - Error Occurred"),
								   NULL,
								   GTK_DIALOG_NO_SEPARATOR,
								   GTK_STOCK_OK,
								   GTK_RESPONSE_ACCEPT,
								   NULL));
	gtk_dialog_set_default_response(dialog, GTK_RESPONSE_ACCEPT);
	g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
	hbox = g_object_new(GTK_TYPE_HBOX,
			    NULL);
	icon = g_object_new(GTK_TYPE_IMAGE,
			    "stock", GTK_STOCK_DIALOG_WARNING,
			    "icon-size", GTK_ICON_SIZE_DIALOG,
			    "xalign", 0.5,
			    "yalign", 1.0,
			    NULL);
	
	markup = g_markup_printf_escaped(_("<b>An error occurred accessing the %s sensors.</b>\n\
Did you forget to load the correct module?  Once the problem\n\
is resolved you will need to reload %s."), id, PACKAGE_NAME);

	label = g_object_new(GTK_TYPE_LABEL,
			     "wrap", TRUE,
			     "use-markup", TRUE,
			     "label", markup,
			     NULL);
	g_free(markup);

	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(icon));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(label));
	gtk_box_pack_start_defaults(GTK_BOX(dialog->vbox), GTK_WIDGET(hbox));
	gtk_widget_show_all(GTK_WIDGET(dialog));

}

/* returns a newly allocated string containing the pango fontsize,
   defaults to medium if error */
gchar *sensors_list_get_font_size(SensorsList *sensors_list) {
	FontSize font_size;

	font_size = panel_applet_gconf_get_int(sensors_list->sensors_applet->applet, FONT_SIZE, NULL);
	switch (font_size) {
	case XX_LARGE:
		return g_strdup(XX_LARGE_TEXT);
	case X_LARGE:
		return g_strdup(X_LARGE_TEXT);
	case LARGE:
		return g_strdup(LARGE_TEXT);
	case MEDIUM:
		return g_strdup(MEDIUM_TEXT);
	case SMALL:
		return g_strdup(SMALL_TEXT);
	case X_SMALL:
		return g_strdup(X_SMALL_TEXT);
	case XX_SMALL:
		return g_strdup(XX_SMALL_TEXT);
	default:
		return g_strdup(MEDIUM_TEXT);
	}
}

/* HELPER FUNCTIONS for managing labels and values lists */
/* removes remainder of list links and frees widgets inside list */
void sensors_list_crop_list(GSList **list, GSList **current_element) {
	while (*current_element != NULL) {
		GSList *old_element = *current_element;
		gtk_widget_destroy(GTK_WIDGET(old_element->data));
		*list = g_slist_remove_link(*list, old_element);
		*current_element = g_slist_next(*current_element);
	}
}

/* take the given text and add if there is already a node at the given
 * element, change its text to this, or create a new node and point
 * current_element at the newly created node */
void sensors_list_add_label_to_list(GSList **list, GSList **current_element, gchar *text) {
	
	GtkLabel *label;

	/* if have reached end of list, need to create a new node etc */
	if (*current_element == NULL) {
		/* create a new label and add it to
		   the list and the display */
		label = g_object_new(GTK_TYPE_LABEL,
				     "label", text,
				     "use-markup", TRUE,
				     "xalign", 0.0,
				     NULL);

		*list = g_slist_append(*list, label);

		/* if we have just created the first
		   node in the list, we will want to be
		   pointing at it now */
		if (g_slist_length(*list) == 1) {
			*current_element = *list;
		} else {
			/* if we just appended a new
			   node, we will want to be
			   pointing at this as the
			   current node */
			*current_element = g_slist_next(*current_element);
			
		}
	} else {
		/* assign text to current element (a GtkLabel) */
		gtk_label_set_markup(GTK_LABEL((*current_element)->data), text);
	}
}

void sensors_list_init(SensorsList *sensors_list) {
	g_assert(sensors_list != NULL);
	g_assert(sensors_list->sensors_applet != NULL);

	sensors_list->list = gtk_list_store_new(N_COLUMNS, 
						G_TYPE_STRING, /* path
								* */

						G_TYPE_STRING, /* filename */

						G_TYPE_STRING, /* label */

						G_TYPE_BOOLEAN, /* enable */

						G_TYPE_DOUBLE, /* alarm value */

						G_TYPE_INT, /* alarm type */
						G_TYPE_BOOLEAN, /* alarm enable */
						G_TYPE_STRING, /* alarm command */ 
						G_TYPE_INT, /* alarm timeout */
						G_TYPE_UINT, /* alarm
							      * timeout
							      index */
						G_TYPE_UINT, /* sensor
							     * type */
						G_TYPE_DOUBLE, /* multiplier */
						G_TYPE_DOUBLE /* offset */);




	sensors_list->view = g_object_new(GTK_TYPE_TREE_VIEW,
					  "model", GTK_TREE_MODEL(sensors_list->list),		     
					  "rules-hint", TRUE,
					  "reorderable", TRUE,
					  "enable-search", TRUE,
					  "search-column", LABEL_COLUMN,
					  NULL);
	
	sensors_list->filename_renderer = gtk_cell_renderer_text_new();
	sensors_list->label_renderer = gtk_cell_renderer_text_new();
	g_object_set(sensors_list->label_renderer,
		     "editable", TRUE,
		     NULL);
	
	g_signal_connect(sensors_list->label_renderer, "edited", G_CALLBACK(sensors_list_sensor_name_changed), sensors_list);

	sensors_list->enable_renderer = gtk_cell_renderer_toggle_new();
	g_signal_connect(sensors_list->enable_renderer, "toggled", G_CALLBACK(sensors_list_sensor_toggled), sensors_list);
	
	sensors_list->filename_column = gtk_tree_view_column_new_with_attributes(_("Filename"),
										 sensors_list->filename_renderer,
										 "text", FILENAME_COLUMN,
										 NULL);
	
	gtk_tree_view_column_set_min_width(sensors_list->filename_column, 90);
	
	sensors_list->label_column = gtk_tree_view_column_new_with_attributes(_("Label"),
									      sensors_list->label_renderer,
									      "text", LABEL_COLUMN,
									      NULL);
	
	gtk_tree_view_column_set_min_width(sensors_list->label_column, 90);
	sensors_list->enable_column = gtk_tree_view_column_new_with_attributes(_("Enabled"),
									       sensors_list->enable_renderer,
									       "active", ENABLE_COLUMN,
									       NULL);
	
	gtk_tree_view_column_set_max_width(sensors_list->enable_column, 50);

	gtk_tree_view_append_column(sensors_list->view, sensors_list->filename_column);
	gtk_tree_view_append_column(sensors_list->view, sensors_list->label_column);
	gtk_tree_view_append_column(sensors_list->view, sensors_list->enable_column);
	sensors_list->scrolled_window = g_object_new(GTK_TYPE_SCROLLED_WINDOW,
						     "hadjustment", NULL, 
						     "height-request", 200,
						     "hscrollbar-policy", GTK_POLICY_AUTOMATIC,
						     "vadjustment",NULL,
						     "vscrollbar-policy", GTK_POLICY_AUTOMATIC,
						     NULL);

	gtk_container_add(GTK_CONTAINER(sensors_list->scrolled_window), GTK_WIDGET(sensors_list->view));

	/* GtkTree Selection */
	sensors_list->selection = gtk_tree_view_get_selection(sensors_list->view);
	/* allow user to only select one row at a time at most */
	gtk_tree_selection_set_mode(sensors_list->selection, GTK_SELECTION_SINGLE);
	/* when selection is changed, make sure sensor_config button is
	   activated */

	/* Sensor Config button */
	/* initially make button insensitive until user selects a row
	   from the sensors_list */
	sensors_list->sensor_config_button = g_object_new(GTK_TYPE_BUTTON,
						  "label", _("Configure _Sensor"),
						  "use-underline", TRUE,
						  "sensitive", FALSE,
						  NULL);
	

	g_signal_connect(sensors_list->selection,
			 "changed",
			 G_CALLBACK(sensors_list_selection_changed),
			 sensors_list);
	
	/* pass selection to signal handler so we can give user a
	   dialog with the selected rows alarm value and enable */
	g_signal_connect(sensors_list->sensor_config_button, "clicked", G_CALLBACK(sensors_list_sensor_config_button_clicked), sensors_list);

	sensors_list->vbox = g_object_new(GTK_TYPE_VBOX,
					  "border-width", 5,
					  "homogeneous", FALSE,
					  "spacing", 0,
					  NULL);

	/* pack sensors_vbox */
	gtk_box_pack_start(sensors_list->vbox, GTK_WIDGET(sensors_list->scrolled_window), FALSE, FALSE, 0);
	sensors_list->hbox = g_object_new(GTK_TYPE_HBOX,
					  "border-width", 5,
					  "homogeneous", FALSE,
					  "spacing", 0,
					  NULL);
	gtk_box_pack_end(sensors_list->hbox, GTK_WIDGET(sensors_list->sensor_config_button), FALSE, FALSE, 0);
	gtk_box_pack_start(sensors_list->vbox, GTK_WIDGET(sensors_list->hbox), FALSE, FALSE, 0);
}


/* gets called if are already setup so we don't have to manually go
   through and find sensors etc again */
void sensors_list_setup_from_gconf(SensorsList *sensors_list) {
	/* sensors_list->list should have been created but should be
	   empty at this stage */

	/* everything gets stored except alarm timeout indexes, which
	   we set to -1 */
	GSList *paths, *filenames, *labels, *enables, *alarm_values, *alarm_types, *alarm_enables, *alarm_commands, *alarm_timeouts, *sensor_types, *multipliers, *offsets;
	GSList *current_path, *current_filename, *current_label, *current_enable, *current_alarm_value, *current_alarm_type, *current_alarm_enable, *current_alarm_command, *current_alarm_timeout, *current_sensor_type, *current_multiplier, *current_offset;
	GtkTreeIter iter;
	
	/* get lists from gconf */
	sensors_list_get_list_from_gconf(sensors_list, &paths, PATHS, GCONF_VALUE_STRING);
	sensors_list_get_list_from_gconf(sensors_list, &filenames, FILENAMES, GCONF_VALUE_STRING);
	sensors_list_get_list_from_gconf(sensors_list, &labels, LABELS, GCONF_VALUE_STRING);
	sensors_list_get_list_from_gconf(sensors_list, &enables, ENABLES, GCONF_VALUE_BOOL);

	/* alarm values get stored as (int)value * 1000 so we don't
	 * lose accuracy */
	sensors_list_get_list_from_gconf(sensors_list, &alarm_values, ALARM_VALUES, GCONF_VALUE_INT);
	sensors_list_get_list_from_gconf(sensors_list, &alarm_types, ALARM_TYPES, GCONF_VALUE_INT);
	sensors_list_get_list_from_gconf(sensors_list, &alarm_enables, ALARM_ENABLES, GCONF_VALUE_BOOL);
	sensors_list_get_list_from_gconf(sensors_list, &alarm_commands, ALARM_COMMANDS, GCONF_VALUE_STRING);
	sensors_list_get_list_from_gconf(sensors_list, &alarm_timeouts, ALARM_TIMEOUTS, GCONF_VALUE_INT);
	sensors_list_get_list_from_gconf(sensors_list, &sensor_types, SENSOR_TYPES, GCONF_VALUE_INT);
	/* multipliers and offsets get stored as (int)value * 1000 so
	 * we don't lose accuracy */
	sensors_list_get_list_from_gconf(sensors_list, &multipliers, MULTIPLIERS, GCONF_VALUE_INT);
	sensors_list_get_list_from_gconf(sensors_list, &offsets, OFFSETS, GCONF_VALUE_INT);


	/* now put lists into sensors_list GtkListStore */	
	for (current_path = paths, current_filename = filenames, current_label = labels, current_enable = enables, current_alarm_value = alarm_values, current_alarm_type = alarm_types, current_alarm_enable = alarm_enables, current_alarm_command = alarm_commands, current_alarm_timeout = alarm_timeouts, current_sensor_type = sensor_types, current_multiplier = multipliers, current_offset = offsets; 
	     current_path != NULL && current_filename != NULL && current_label != NULL && current_enable != NULL && current_alarm_value != NULL && current_alarm_type != NULL && current_alarm_enable != NULL && current_alarm_command != NULL && current_alarm_timeout != NULL && current_sensor_type != NULL && current_multiplier != NULL && current_offset != NULL; 
	     current_path = g_slist_next(current_path), current_filename = g_slist_next(current_filename), current_label = g_slist_next(current_label), current_enable = g_slist_next(current_enable), current_alarm_value = g_slist_next(current_alarm_value), current_alarm_type = g_slist_next(current_alarm_type), current_alarm_enable = g_slist_next(current_alarm_enable), current_alarm_command = g_slist_next(current_alarm_command), current_alarm_timeout = g_slist_next(current_alarm_timeout), current_sensor_type = g_slist_next(current_sensor_type), current_multiplier = g_slist_next(current_multiplier), current_offset = g_slist_next(current_offset)) {
		gtk_list_store_append(sensors_list->list, &iter);

		gtk_list_store_set(sensors_list->list,
				   &iter,
				   PATH_COLUMN, (gchar *)current_path->data,
				   FILENAME_COLUMN, (gchar *)current_filename->data,
				   LABEL_COLUMN, (gchar *)current_label->data,
				   ENABLE_COLUMN, GPOINTER_TO_INT(current_enable->data),
				   ALARM_VALUE_COLUMN, (gdouble)GPOINTER_TO_INT(current_alarm_value->data) / 1000.0,
				   ALARM_TYPE_COLUMN, GPOINTER_TO_INT(current_alarm_type->data),
				   ALARM_ENABLE_COLUMN, GPOINTER_TO_INT(current_alarm_enable->data),
				   ALARM_COMMAND_COLUMN, (gchar *)current_alarm_command->data,
				   ALARM_TIMEOUT_COLUMN, GPOINTER_TO_INT(current_alarm_timeout->data),
				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
				   SENSOR_TYPE_COLUMN, GPOINTER_TO_INT(current_sensor_type->data),
				   MULTIPLIER_COLUMN, (gdouble)GPOINTER_TO_INT(current_multiplier->data) / 1000.0,
				   OFFSET_COLUMN, (gdouble)GPOINTER_TO_INT(current_offset->data) / 1000.0,

				   -1);
	}

	g_slist_free(paths);
	g_slist_free(filenames);
	g_slist_free(labels);
	g_slist_free(enables);
	g_slist_free(alarm_values);
	g_slist_free(alarm_types);
	g_slist_free(alarm_enables);
	g_slist_free(alarm_commands);
	g_slist_free(alarm_timeouts);
	g_slist_free(sensor_types);
	g_slist_free(multipliers);
	g_slist_free(offsets);
}

gboolean sensors_list_gconf_paths_valid(SensorsApplet *sensors_applet, const gchar *id) {
	GSList *paths = NULL;
	GSList *current_path;
	gchar *paths_key;
	FILE *fp;
	
	paths_key = g_strdup_printf("%s/%s", id, PATHS);
	g_assert(paths = panel_applet_gconf_get_list(sensors_applet->applet,
					    paths_key,
					    GCONF_VALUE_STRING,
					    NULL));
	g_free(paths_key);
	for (current_path = paths; current_path != NULL; current_path = g_slist_next(current_path)) {
		if (fp = fopen((gchar *)current_path->data, "r")) {
			fclose(fp);
		} else {
			g_slist_foreach(paths, (GFunc)g_free, NULL);
			g_slist_free(paths);
			return FALSE;
		}
	}
	g_slist_foreach(paths, (GFunc)g_free, NULL);
	g_slist_free(paths);
	return TRUE;
}

void sensors_list_write_to_gconf(SensorsList *sensors_list) {
	/* write paths, filenames, labels, enables AND setup to gconf */

	/* for stepping through GtkListStore */
	GtkTreeIter iter;
	gboolean not_end_of_list = TRUE;

	GSList *paths = NULL, *filenames = NULL, *labels = NULL, *enables = NULL, *alarm_values = NULL, *alarm_types = NULL, *alarm_enables = NULL, *alarm_commands = NULL, *alarm_timeouts = NULL, *sensor_types = NULL, *multipliers = NULL, *offsets = NULL;
	gchar *current_path, *current_filename, *current_label, *current_alarm_command;
	gboolean current_enable, current_alarm_enable;
	gdouble current_alarm_value, current_multiplier, current_offset;
	gint current_alarm_timeout, current_sensor_type, current_alarm_type;

	GError *error = NULL;
	/* now step through the GtkListStore sensors_list to
	   store each value in gconf as a list */
	for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_list->list), &iter); not_end_of_list; not_end_of_list = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_list->list), &iter)) {
		gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list), 
				   &iter,
				   PATH_COLUMN, &current_path,
				   FILENAME_COLUMN, &current_filename,
				   LABEL_COLUMN, &current_label,
				   ENABLE_COLUMN, &current_enable,
				   ALARM_VALUE_COLUMN, &current_alarm_value,
				   ALARM_TYPE_COLUMN, &current_alarm_type,
				   ALARM_ENABLE_COLUMN, &current_alarm_enable,
				   ALARM_COMMAND_COLUMN, &current_alarm_command,
				   ALARM_TIMEOUT_COLUMN, &current_alarm_timeout,
				   SENSOR_TYPE_COLUMN, &current_sensor_type,
				   MULTIPLIER_COLUMN, &current_multiplier,
				   OFFSET_COLUMN, &current_offset,
				   -1);
		paths = g_slist_append(paths, current_path);
		filenames = g_slist_append(filenames, current_filename);
		labels = g_slist_append(labels, current_label);
		enables = g_slist_append(enables, GINT_TO_POINTER(current_enable));
		alarm_values = g_slist_append(alarm_values, GINT_TO_POINTER((gint)(current_alarm_value * 1000)));
		alarm_types = g_slist_append(alarm_types, GINT_TO_POINTER(current_alarm_type));
		alarm_enables = g_slist_append(alarm_enables, GINT_TO_POINTER(current_alarm_enable));
		alarm_commands = g_slist_append(alarm_commands, current_alarm_command);
		alarm_timeouts = g_slist_append(alarm_timeouts, GINT_TO_POINTER(current_alarm_timeout));
		sensor_types = g_slist_append(sensor_types, GINT_TO_POINTER(current_sensor_type));
		multipliers = g_slist_append(multipliers, GINT_TO_POINTER((gint)(current_multiplier * 1000)));
		offsets = g_slist_append(offsets, GINT_TO_POINTER((gint)(current_offset * 1000)));


	}

	sensors_list_set_list_in_gconf(sensors_list, paths, PATHS, GCONF_VALUE_STRING);
	sensors_list_set_list_in_gconf(sensors_list, filenames, FILENAMES, GCONF_VALUE_STRING);
	sensors_list_set_list_in_gconf(sensors_list, labels, LABELS, GCONF_VALUE_STRING);
	sensors_list_set_list_in_gconf(sensors_list, enables, ENABLES, GCONF_VALUE_BOOL);
	sensors_list_set_list_in_gconf(sensors_list, alarm_values, ALARM_VALUES, GCONF_VALUE_INT);
	sensors_list_set_list_in_gconf(sensors_list, alarm_types, ALARM_TYPES, GCONF_VALUE_INT);
	sensors_list_set_list_in_gconf(sensors_list, alarm_enables, ALARM_ENABLES, GCONF_VALUE_BOOL);
	sensors_list_set_list_in_gconf(sensors_list, alarm_commands, ALARM_COMMANDS, GCONF_VALUE_STRING);
	sensors_list_set_list_in_gconf(sensors_list, alarm_timeouts, ALARM_TIMEOUTS, GCONF_VALUE_INT);
	sensors_list_set_list_in_gconf(sensors_list, sensor_types, SENSOR_TYPES, GCONF_VALUE_INT);
	sensors_list_set_list_in_gconf(sensors_list, multipliers, MULTIPLIERS, GCONF_VALUE_INT);
	sensors_list_set_list_in_gconf(sensors_list, offsets, OFFSETS, GCONF_VALUE_INT);


	/* need to free allocated strings */
	g_slist_foreach(paths, (GFunc)g_free, NULL);
	g_slist_free(paths);

	g_slist_foreach(filenames, (GFunc)g_free, NULL);
	g_slist_free(filenames);

	g_slist_foreach(labels, (GFunc)g_free, NULL);
	g_slist_free(labels);
	g_slist_free(enables);
	g_slist_free(alarm_values);
	g_slist_free(alarm_types);
	g_slist_free(alarm_enables);
	g_slist_foreach(alarm_commands, (GFunc)g_free, NULL);
	g_slist_free(alarm_commands);
	g_slist_free(alarm_timeouts);
	g_slist_free(sensor_types);
	g_slist_free(multipliers);
	g_slist_free(offsets);

	gchar *key = g_strdup_printf("%s/%s", sensors_list->id, IS_SETUP);
	panel_applet_gconf_set_bool(sensors_list->sensors_applet->applet,
				    key,
				    TRUE,
				    NULL);
	g_free(key);
}

void sensors_list_alarm_on(SensorsList *sensors_list, GtkTreeIter *iter) {
	guint alarm_timeout_index;
	gchar *alarm_command;
	gint alarm_timeout;
	gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list),
			   iter,
			   ALARM_COMMAND_COLUMN, &alarm_command,
			   ALARM_TIMEOUT_COLUMN, &alarm_timeout,
			   ALARM_TIMEOUT_INDEX_COLUMN, &alarm_timeout_index,
			   -1);

	if (alarm_timeout_index == -1) {
		/* alarm is not currently on */
		if (alarm_timeout <= 0) {
			gnome_execute_shell(NULL, alarm_command);

			alarm_timeout_index = g_timeout_add(G_MAXINT,
						    (GSourceFunc)sensors_list_alarm_timeout,
						    alarm_command);
		} else {
			alarm_timeout_index = g_timeout_add(alarm_timeout * 1000,
							    (GSourceFunc)sensors_list_alarm_timeout,
							    alarm_command);
		}
		gtk_list_store_set(sensors_list->list,
				   iter,
				   ALARM_TIMEOUT_INDEX_COLUMN, alarm_timeout_index,
				   -1);
		
	} 
}

void sensors_list_alarm_off(SensorsList *sensors_list, GtkTreeIter *iter) {
	guint alarm_timeout_index;
	gtk_tree_model_get(GTK_TREE_MODEL(sensors_list->list),
			   iter,
			   ALARM_TIMEOUT_INDEX_COLUMN, &alarm_timeout_index,
			   -1);
	if (alarm_timeout_index != -1) {
		/* alarm is on */
		if (!g_source_remove(alarm_timeout_index)) {
			g_printf("Error removing alarm source\n");
		}
		gtk_list_store_set(sensors_list->list,
				   iter,
				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
				   -1);
	}
}
