/*
 * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Structure to hold pointers to a GtkListStore, GtkTreeView, Columns
 * and GtkTreeSelection for a list of sensors (i2c etc) and a scrolled
 * window to place them into so if we need we can access them from
 * anywhere in the program via the SensorsApplet struct */

#ifndef SENSORS_LIST_H
#define SENSORS_LIST_H

#include "sensors-applet.h"

typedef struct _SensorsList SensorsList;
typedef void (*UpdateLabelsFunction)(SensorsList *sensors_list);

struct _SensorsList {
	SensorsApplet *sensors_applet;

	const gchar *id; /* "i2c" (or "i8k" in future */
	GtkListStore *list; /* the actual data structure */
	
	/* pointer to function to update labels list etc */
	UpdateLabelsFunction update_labels;

	/* GUI widgets etc used in the prefs dialog */
	GtkTreeView *view; 
	GtkTreeSelection *selection;
	GtkTreeViewColumn *filename_column, *label_column, *enable_column;
	GtkCellRenderer *filename_renderer, *label_renderer, *enable_renderer;
	GtkScrolledWindow *scrolled_window;
	GtkBox *vbox; /* holds scrolled_window and hbox */
	GtkBox *hbox; /* holds alarm button */
	GtkButton *sensor_config_button;

	GSList *labels;
	GSList *values;
};


/* enumeration used to identify columns in the GtkListStore list */
enum {
	PATH_COLUMN = 0,
	FILENAME_COLUMN,
	LABEL_COLUMN,
	ENABLE_COLUMN,
       	ALARM_VALUE_COLUMN,
	ALARM_TYPE_COLUMN, 
	ALARM_ENABLE_COLUMN,
	ALARM_COMMAND_COLUMN,
	ALARM_TIMEOUT_COLUMN,
	ALARM_TIMEOUT_INDEX_COLUMN,
	SENSOR_TYPE_COLUMN,
	MULTIPLIER_COLUMN,
	OFFSET_COLUMN,
	N_COLUMNS
};

/* alarm types */
typedef enum {
	ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD = 0,
	ALARM_WHEN_VALUE_LESS_THAN_THRESHOLD
} AlarmType;

/* function prototypes */

void sensors_list_init(SensorsList *sensors_list);
void sensors_list_gconf_error_occurred(const gchar *id);
gboolean sensors_list_gconf_paths_valid(SensorsApplet *sensors_applet, const gchar *id);
void sensors_list_setup_from_gconf(SensorsList *sensors_list);
void sensors_list_add_label_to_list(GSList **list, GSList **current_element, gchar *text);
void sensors_list_crop_list(GSList **list, GSList **current_element);
gchar *sensors_list_get_font_size(SensorsList *sensors_list);

#endif /* SENSORS_LIST_H */
